@@ -21,6 +21,10 @@ std::array<const char*, TPL_NY> observableNames = {
     TPL_OBSERVABLE_NAMES_INITIALIZER_LIST
 };
 
+std::array<const char*, TPL_NW> expressionNames = {
+    TPL_EXPRESSION_NAMES_INITIALIZER_LIST
+};
+
 std::array<const char*, TPL_NP> parameterIds = {
     TPL_PARAMETER_IDS_INITIALIZER_LIST
 };
@@ -37,6 +41,9 @@ std::array<const char*, TPL_NY> observableIds = {
     TPL_OBSERVABLE_IDS_INITIALIZER_LIST
 };
 
+std::array<const char*, TPL_NW> expressionIds = {
+    TPL_EXPRESSION_IDS_INITIALIZER_LIST
+};
 
 } // namespace model_TPL_MODELNAME
 
