@@ -374,6 +374,10 @@ bool SteadystateProblem::getSensitivityFlag(const Model *model,
         steady_state_status_[1] == SteadyStateStatus::success &&
         model->getSteadyStateSensitivityMode() == SteadyStateSensitivityMode::simulationFSA;
 
+    bool simulationStartedInSteadystate =
+        steady_state_status_[0] == SteadyStateStatus::success &&
+        numsteps_[0] == 0;
+
     /* Do we need forward sensis for postequilibration? */
     bool needForwardSensisPosteq = !preequilibration &&
         !forwardSensisAlreadyComputed &&
@@ -388,7 +392,8 @@ bool SteadystateProblem::getSensitivityFlag(const Model *model,
 
     /* Do we need to do the linear system solve to get forward sensitivities? */
     bool needForwardSensisNewton =
-        needForwardSensisPreeq || needForwardSensisPosteq;
+        (needForwardSensisPreeq || needForwardSensisPosteq) &&
+        !simulationStartedInSteadystate;
 
     /* When we're creating a new solver object */
     bool needForwardSensiAtCreation = needForwardSensisPreeq &&
@@ -400,7 +405,9 @@ bool SteadystateProblem::getSensitivityFlag(const Model *model,
             return needForwardSensisNewton;
 
         case SteadyStateContext::sensiStorage:
-            return needForwardSensisNewton || forwardSensisAlreadyComputed;
+            return needForwardSensisNewton ||
+                forwardSensisAlreadyComputed ||
+                simulationStartedInSteadystate;
 
         case SteadyStateContext::solverCreation:
             return needForwardSensiAtCreation;
